"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ix_1 = require("@reactive-js/ix");
const rx_1 = require("@reactive-js/rx");
const event_1 = require("./event");
const pipe_1 = require("@reactive-js/pipe");
const locationEquals = (a, b) => a === b ||
    (a.path === b.path && a.query === b.query && a.fragment === b.fragment);
const chopLeadingChar = (str) => str.length > 0 ? str.substring(1) : "";
const getCurrentLocation = (_) => {
    const path = window.location.pathname;
    const query = chopLeadingChar(window.location.search);
    const fragment = chopLeadingChar(window.location.hash);
    return { path, query, fragment };
};
const pushHistoryState = (newLocation) => {
    const currentLocation = getCurrentLocation();
    if (!locationEquals(currentLocation, newLocation)) {
        const { path, query, fragment } = newLocation;
        let uriString = path;
        uriString = query.length > 0 ? `${uriString}?${query}` : uriString;
        uriString = fragment.length > 0 ? `${uriString}#${fragment}` : uriString;
        window.history.pushState(undefined, "", uriString);
    }
};
const historyOperator = (obs) => rx_1.merge(rx_1.ofValue(getCurrentLocation()), pipe_1.pipe(obs, rx_1.onNext(pushHistoryState)), event_1.fromEvent(window, "popstate", getCurrentLocation));
const _historyIterable = {
    getIXAsyncIterator(scheduler, replayCount) {
        return ix_1.createAsyncIteratorResource(historyOperator, scheduler, replayCount);
    },
};
exports.historyIterable = _historyIterable;
//# sourceMappingURL=historyIterable.js.map